# -*- coding: utf-8 -*-
"""
Opérations simples sur les matrices : calcul de rang, d'inverse...

"""

import pivot_corrige as pivot
from copy import deepcopy


def rang(A):
    """
    Calcule le rang de A et le retourne
    """
    C = deepcopy(A)  # on travaille maintenant avec C pour éviter de modifier A
    pivot.echelonne(C)
    l = len(C) - 1
    while l >= 0 and pivot.cherche_pivot_ligne(C, l) is None:
        l -= 1
    return l + 1  # l est le numéro de la dernière ligne non nulle


def identite(n):
    """
    Cré la matrice identité de taille n
    """
    A = []
    for i in range(n):
        l = []
        for j in range(n):
            if i == j:
                l.append(1)
            else:
                l.append(0)
        A.append(l)
    return A
# Méthode 2, moins lisible
# return [[1 if i == j else 0 for j in range(n)] for i in range(n)]


def inverse(A):
    """
    Retourne l'inverse de A
    """
    if (len(A) != len(A[0])):
        print('Matrice non carrée')
        return
    C = deepcopy(A)
    B = identite(len(A))
    pivot.echelonne(C, B)
    # print(C)
    pivot.reduit(C, B)
    return B


def colonne_nulle(n):
    """
    retourne une colonne nulle de taille n
    """
    return [[0] for i in range(n)]


def solutions(A, B):
    """
    Retourne les solutions de AX = B.

    S'il n'y a pas de solutions, retourne None
    S'il y a une unique solution, retourne la matrice colonne correspondante.
    S'il y a une infinité de solution, retourne [Xp, [C1, ..., Cp]] où Xp est une solution
    et Vect(C1, ..., Cp) est l'ensemble des solutions du système homogène.
    """
    C = deepcopy(A)
    D = deepcopy(B)
    pivot.echelonne(C, D)
    l = len(C) - 1
    while l >= 0 and pivot.cherche_pivot_ligne(C, l) is None:
        # vérification que le système est compatible
        if D[l][0] != 0:
            return None
        l = l - 1
    r = l + 1  # le rang
    pivot.reduit(C, D)
    if r == len(A) and r == len(A[0]):
        return C
    # liste des indices de colonnes des pivots, ie des indices des inconnues principales.
    # on l'obtient par ordre strictement décroissant
    principales = []
    parametres = []
    n = len(C[0])  # nombre d'inconnues
    dernier = n - 1  # indice de la dernière inconnue traités
    while l >= 0:
        indice = pivot.cherche_pivot_ligne(C, l)
        principales.append(indice)
        # ajout des éventuels paramètres entre 2 inconnues principales
        parametres.extend(range(dernier, indice, -1))
        dernier = indice - 1
        l = l - 1

    # renverser les listes construites pour rencontrer les parametres et pivots dans le bon ordre
    parametres.reverse()
    principales.reverse()
    # remplissage des vecteurs
    Xp = colonne_nulle(n)
    vect = [colonne_nulle(n) for j in range(len(parametres))]
    for i in range(len(principales)):
        p = principales[i]  # numéro de l'inconnue principale
        for j in range(len(parametres)):
            q = parametres[j]  # numéro du paramètre courant
            vect[j][p] = [-C[i][q]]
        Xp[p] = D[i]  # la constante se trouve dans la ligne i
    for j in range(len(parametres)):
        q = parametres[j]
        vect[j][q] = [1]
    return (Xp, vect)
