# -*- coding: utf-8 -*-
"""
Opérations élémentaires sur les matrices
"""


def echange(A, i, j):
    """
    A est une matrice

    Echange les lignes i et j dans A.
    """
    A[i], A[j] = A[j], A[i]


def multiplie(A, i, k):
    """
    Multiplie la lige i de la matrice A par une constante k non nulle.
    """
    if k != 0:
        for j in range(len(A[i])):
            A[i][j] = k * A[i][j]


def combinaison(A, i, j, k):
    """
    Ajoute k fois la ligne j à la ligne i dans A.
    i doit être différent de j
    """
    if i != j:
        for l in range(len(A[i])):
            A[i][l] = A[i][l] + k * A[j][l]


def cherche_pivot_colonne(A, j, ligne_min):
    """
    Trouve un pivot dans la colonne j de A en dessous (<=) de ligne_min.

    Retourne l'indice de la ligne trouvée ou False si tous les coefficients de
    la colonne sont nuls.
    """
    for i in range(ligne_min, len(A)):
        if A[i][j] != 0:
            return i
    return None


def echelonne(A, B=None):
    """
    Echelonne la matrice A par l'algorithme du pivot de Gauss.
    Reporte les opérations sur la matrice B si une matrice est donnée.
    """
    l, c = 0, 0  # ligne et colonne courante
    while (l < len(A) and c < len(A[0])):
        p = cherche_pivot_colonne(A, c, l)
        if p is not None:  # tous les coefficients de cette colonne sont nuls
            if p != l:
                echange(A, p, l)
                if B is not None:
                    echange(B, p, l)
            for i in range(l + 1, len(A)):
                # modification de chacune des lignes suivantes
                if B is not None:
                    combinaison(B, i, l, -A[i][c] / A[l][c])
                combinaison(A, i, l, -A[i][c] / A[l][c])

            l += 1
        c += 1
    return A, B


def cherche_pivot_ligne(A, i):
    """
    Trouve l'indice du premier coefficient non nul dans la ligne i de la matrice A

    Retourne cet indice ou False si la ligne est nulle.
    """
    for j in range(len(A[i])):
        if A[i][j] != 0:
            return j
    return None


def reduit(A, B=None):
    """
    Reduit la matrice échelonnée A et reporte les opération sur B.
    """
    l = len(A) - 1
    while l > 0:
        c = cherche_pivot_ligne(A, l)
        if c is not None:
            for i in range(0, l):  # pour les lignes au dessus
                # d'abord B avant de modifier le coefficient de A...
                if B is not None:
                    combinaison(B, i, l, -A[i][c] / A[l][c])
                combinaison(A, i, l, -A[i][c] / A[l][c])
            if B is not None:
                multiplie(B, l, 1 / A[l][c])
            multiplie(A, l, 1 / A[l][c])
        l -= 1
    return A, B
